#include "umf/umf_tool.lua"
#include "bubblinebee/weapon.lua"

--[[

	AXIOM EXTRASOLAR INCORPORATED PRESENTS
	HUNTRESS SCATTERGUN
	
	(Mod by Bubblinebee)

]]

local TOOL = {}
local pierceHandler = ShellHandler:new()
local scatterHandler = ShellHandler:new()

TOOL.printname = "Huntress Scattergun"
TOOL.model = "vox/scattergun.vox"
TOOL.damageState = 0 --Damage state: 0 - Kinetic, 1 - Energy, 2 - Explosive
TOOL.animationTimer = 0.0
TOOL.fireSound = nil

function TOOL:Initialize(dt)
	pierceHandler:Initialize({maxShells = 32})
	scatterHandler:Initialize({maxShells = 2048})

	self.fireSound = LoadSound("MOD/snd/fire.ogg")
end

function TOOL:Tick(dt)
	pierceHandler:SimulateShells(dt, onPierceHit, onPierceSim, onPierceExpire)
	scatterHandler:SimulateShells(dt, onScatterHit, onScatterSim, nil)

	local b = GetToolBody()
	if b ~= 0 then
			local offset = VecScale(Vec(0, 0, 0.5), self.animationTimer)
			SetToolTransform(Transform(offset))

			self.animationTimer = self.animationTimer + (2.0 * dt) * -self.animationTimer
	end

	if InputPressed("r") then
		self.damageState = (self.damageState + 1) % 3
	end
end

function TOOL:Draw(dt)
	UiPush()
		UiColor(0.85, 0.85, 0.85)
		UiAlign("middle")
		UiFont("regular.ttf", 26)
		UiTextOutline(0,0,0,1,0.2)

		UiTranslate(20, (UiHeight()/1.2))
		UiPush()
			if self.damageState == 0 then
				UiText("Kinetic")
			elseif self.damageState == 1 then
				UiText("Energy")
			elseif self.damageState == 2 then
				UiText("Explosive")
			end
		UiPop()			
	UiPop()
end

function TOOL:LeftClick()
	local toolTrans = GetBodyTransform(GetToolBody())
	local toolPos = TransformToParentPoint(toolTrans, Vec(0.35, -0.6, -2.1))
	local aim = TransformToParentPoint(toolTrans, Vec(0.35, -0.6, -3.1))
	local dir = VecNormalize(VecSub(aim, toolPos))
	local shell = {}

	shell.position = toolPos
	shell.lifetime = 4
	shell.velocity = VecScale(dir, 40.0)
	shell.doGravity = false
	shell.size = 0.25
	shell.damageState = self.damageState

	pierceHandler:AddShell(shell)
	self.animationTimer = 1.0
	PlaySound(self.fireSound)
end

function onPierceHit(shell, hitPos, normal, shape, dt)
	MakeHole(hitPos, 1.0, 0.95, 0.9)
	MakeHole(hitPos, 1.0, 0.95, 0.9)
	return true
end

function onPierceSim(shell, dt)
	ParticleReset()
	ParticleRadius(0.1, 0.01)
	ParticleType("smoke")
	SpawnParticle(shell.position, Vec(0, 0, 0), 1.0)
	if InputDown("rmb") then
		return onPierceExpire(shell, dt)
	else
		return true
	end
end

function onPierceExpire(shell, dt)
	if (shell.damageState == 0) then
		Explosion(shell.position, 0.25)
	elseif (shell.damageState == 1) then
		Explosion(shell.position, 0.25)
	elseif (shell.damageState == 2) then
		Explosion(shell.position, 0.5)
	end

	for i=1, 64 do
		local point2 = VecAdd(shell.position, VecScale(shell.velocity, dt))
		local dir = VecNormalize(VecSub(point2, shell.position))
		local perp = getPerpendicular(dir)
		local spread = 0.25 * rnd(0.0, 1.0)
		dir = VecNormalize(VecAdd(dir, VecScale(perp, spread)))
		local scatterShell = {}

		scatterShell.position = shell.position
		scatterShell.lifetime = 1
		scatterShell.velocity = VecScale(dir, 80.0)
		scatterShell.doGravity = false
		if (shell.damageState == 0) then
			scatterShell.pierce = 3
			scatterShell.explode = false
			scatterShell.burn = false
			scatterShell.color = {1, 0.25, 0, 0.9, 0.1, 0.1}
		elseif (shell.damageState == 1) then
			scatterShell.pierce = 1
			scatterShell.explode = false
			scatterShell.burn = true
			scatterShell.color = {0, 0.25, 1, 0.1, 0.1, 0.9}
		elseif (shell.damageState == 2) then
			scatterShell.pierce = 1
			scatterShell.explode = true
			scatterShell.burn = false
			scatterShell.color = {1, 0.4, 0.2, 0.7, 0.5, 0.5}
		end

	
		scatterHandler:AddShell(scatterShell)
	end
	return false
end

function onScatterHit(shell, hitPos, normal, shape, dt)
	shell.pierce = shell.pierce - 1
	if shell.pierce > 0 then
		if shell.explode then
			Explosion(hitPos, 0.5)
		else
			MakeHole(hitPos, 0.75, 0.7, 0.65)
		end
		if shell.burn then
			local hit, point, n = GetShapeClosestPoint(shape, hitPos)
			if hit then
				SpawnFire(point)
			end
		end
		return true
	else
		if shell.explode then
			Explosion(hitPos, 0.5)
		else
			MakeHole(hitPos, 0.75, 0.7, 0.65)
		end
		if shell.burn then
			local hit, point, n = GetShapeClosestPoint(shape, hitPos)
			if hit then
				SpawnFire(point)
			end
		end
		return false
	end
end

function onScatterSim(shell, dt)
	ParticleReset()
	ParticleRadius(0.05, 0.01)
	ParticleColor(unpack(shell.color))
	ParticleEmissive(5, 0)
	SpawnParticle(shell.position, Vec(0, 0, 0), 1.0)
	return true
end

function getPerpendicular(dir)
	local perp = VecNormalize(Vec(rnd(-1, 1), rnd(-1, 1), rnd(-1, 1)))
	perp = VecNormalize(VecSub(perp, VecScale(dir, VecDot(dir, perp))))
	return perp
end

function rnd(mi, ma)
	local v = math.random(0,1000) / 1000
	return mi + (ma-mi)*v
end

RegisterToolUMF("scattergun", TOOL)